## filter CTC genes based on their RPMs
## Qike Li 12/15/2015

## set working directory
setwd("~/Dropbox/CTCs/CTC_EDA/PID")
rm(list=ls())

######## ######## ######## ######## ######## ######## 
### Read in datat file
## GEO# GSE67980, Miyamoto et al.
## RNA-Seq of single prostate CTCs implicates noncanonical Wnt signaling in antiandrogen resistance
ctc.data <- read.delim(file="~/Dropbox/CTCs/GSE67980/GSE67980_readCounts.txt")

## Read in sample annotations
sample.data <- read.delim(file="~/Dropbox/CTCs/GSE67980/GSE67980_sampleProperties.txt")
row.names(sample.data) <- sample.data$title

## Retrieve the 77 lineage-confirmed PCa CTCs.
linCon.samp <- as.character(sample.data[grep("lineage-confirmed",sample.data$source.name),"title"])
linCon.data <- ctc.data[,c(names(ctc.data)[1:5],linCon.samp)]

ctc.count = linCon.data
dim(ctc.count)


## produce matrix for ease of input to algorithms (and to write to file)
gene.symbols <- as.character(ctc.count$symbol)
ctc.count <- as.matrix(ctc.count[,-(1:5)])
rownames(ctc.count) <- gene.symbols

### create gene row labels 
## remove rows without gene symbols
if (any((rownames(ctc.count) == ""))) {
  ctc.count <- ctc.count[!(rownames(ctc.count) == ""),]
}

## remove duplicate genes by taking max expression
anyDuplicated(rownames(ctc.count)) ## returns first duplicated index
dup.genes <- rownames(ctc.count)[duplicated(rownames(ctc.count))]
length(unique(dup.genes)) ## 33 duplicated genes
## take the max expression
for (ii in unique(dup.genes)) {
  dup.index <- which(rownames(ctc.count) == ii)
  expr.max <- apply(ctc.count[dup.index,],2,max)
  ## fill first occurance with max value
  ctc.count[dup.index[1],] <- expr.max
  ## remove other occurances
  ctc.count <- ctc.count[-dup.index[-1],]
}
anyDuplicated(rownames(ctc.count))
head(ctc.count)
dim(ctc.count)

# save count data
write.table(ctc.count, file = '~/Dropbox/CTCs/CTC_EDA/GSEA/data/ctc.77.allGenes.count.txt', quote = F, sep = '\t')
ctc.rpm = read.delim('~/Dropbox/CTCs/CTC_EDA/ctc.77.allgenes.rpm.txt') # untransformed rpm data
dim(ctc.rpm)
colSums(ctc.rpm)
### Filter as in Miyamoto et al.:
## Filter out genes for which the 0.9 quantile of RPM values was less than 10
threshold <- 10
perc <- 0.9
toKeep.logic <- apply(ctc.rpm,1,function(x) {quantile(x,perc) > threshold})
write.table(ctc.rpm[toKeep.logic, ], file = '~/Dropbox/CTCs/CTC_EDA/GSEA/data/ctc.77.higRPM.rpm.txt', quote = F, sep = '\t')

## group the expression data by paitents. i.e. if a patient has multiple CTCs, we average the expression from all CTCs for each gene.
# define a function to group the expression data
f.groupByPat = function(data1){
  pat.id = substr(colnames(data1), start = 1, stop = regexpr('\\.',colnames(data1))-1)
  pat.ls = split(1:length(pat.id), f = pat.id)
  df.pat.expr = data.frame(row.names = rownames(data1))
  for(i in names(pat.ls)){
    if(length(pat.ls[[i]]) != 1){
      df.pat.expr[[i]] = rowMeans(data1[,pat.ls[[i]]])
    }else{
      df.pat.expr[[i]] = data1[,pat.ls[[i]]]
    }
  }
  return(df.pat.expr)
}
# group ctc rpm data
ctc.rpm.byPat = f.groupByPat(data1 = ctc.rpm)
write.table(ctc.rpm.byPat[toKeep.logic, ], file = '~/Dropbox/CTCs/CTC_EDA/GSEA/data/ctc.13pat.higRPM.rpm.txt',quote = F, sep = '\t')
# group ctc count data
ctc.count.byPat = f.groupByPat(data1 = ctc.count)
write.table(ctc.count.byPat[toKeep.logic, ], file = '~/Dropbox/CTCs/CTC_EDA/DEG_Enrichment/data/ctc.13pat.higRPM.count.txt', quote = F, sep = '\t')


####### find gene length info for all the genes ######################
## the fllowing function is obtained from https://www.biostars.org/p/62583/
library(TxDb.Hsapiens.UCSC.hg19.knownGene) 
library(org.Hs.eg.db)
exons.db = exonsBy(TxDb.Hsapiens.UCSC.hg19.knownGene, by='gene') 
hg19GeneLengths <- function(symbols)
{
  egs    = unlist(  mget(symbols[ symbols %in% keys(org.Hs.egSYMBOL2EG) ],org.Hs.egSYMBOL2EG) )
  sapply(egs,function(eg)
  {
    exons = exons.db[[eg]]
    if(!is.null(exons)){
      exons = reduce(exons)
      sum( width(exons) )
    } else Inf
    
  })
}

# get the gene lengths for every gene
gene.length = hg19GeneLengths(symbols = rownames(ctc.rpm))
df.length = data.frame(Gene = names(gene.length), Length = as.numeric(unlist(gene.length)),row.names = NULL)
df.length = df.length[which(!is.infinite(df.length$Length)),]

# write out df.length
write.table(df.length,file = '~/Dropbox/CTCs/CTC_EDA/GSEA/data/gene_length.txt', quote = F, sep = '\t')


######################################################################

#data.length = read.delim('~/Dropbox/Lab-Tools/Genes/gene.lengths.HG19.txt')

# keep the high RPM genes
dim(ctc.highRPM <- ctc.rpm[toKeep.logic,])
# remove the genes without length info
dim(ctc.highRPM <- ctc.highRPM[rownames(ctc.highRPM) %in% df.length$Gene,])
# divide the rpm data of each gene by gene length to get rpkm
ctc.highRPM.rpkm = ctc.highRPM / df.length[match(rownames(ctc.highRPM), df.length$Gene),]$Length

##convert rpm data of all genes to rpkm data
dim(ctc.rpm)
# remove the genes without length info
dim(ctc.rpm <- ctc.rpm[rownames(ctc.rpm) %in% df.length$Gene,])
# divide the rpm data of each gene by gene length to get rpkm
ctc.rpkm = ctc.rpm/df.length[match(rownames(ctc.rpm), df.length$Gene),]$Length

#write out the rpkm data of all genes
write.table( ctc.rpkm, file = '~/Dropbox/CTCs/CTC_EDA/GSEA/data/ctc.77.allGenes.rpkm.txt',quote = F, sep = '\t')

#write out the rpkm data of the genes with high rpm
write.table(ctc.highRPM.rpkm, file = '~/Dropbox/CTCs/CTC_EDA/GSEA/data/ctc.77.highRPM.rpkm.txt', quote = F, sep = '\t')

# # find the patients who has the enzalutamide status 
# pat.group = sample.data[nchar(as.character(sample.data[,"characteristics..enzalutamide"])) !=0 ,c('characteristics..donor',"characteristics..enzalutamide")]
# # find the patient IDs in native group
# pat.n = pat.group$characteristics..donor[grepl('Group A',pat.group$characteristics..enzalutamide)]
# # find the patient IDs in resistent group
# pat.r = pat.group$characteristics..donor[grepl('Group B',pat.group$characteristics..enzalutamide)]
# # extract out the patient IDs from the column names of ctc.count.highRPM
# colname.patID = substr(colnames(ctc.count.highRPM), start = 1, stop = sapply(gregexpr('\\.', colnames(ctc.count.highRPM)), function(x) x[1])-1)
# # index the naive patient
# (colname.patID %in% pat.n)
# (colname.patID %in% pat.r)
# gregexpr('\\.', colnames(ctc.count.highRPM))[[1]]
# colnames(ctc.count.highRPM)
